import json
from tornado.httpclient import HTTPRequest, AsyncHTTPClient
from tornado.gen import coroutine


@coroutine
def fetch_coroutine(url, method='GET', body=None, **kwargs):
    request = HTTPRequest(url, method=method, body=body, connect_timeout=5, request_timeout=10)
    http_client = AsyncHTTPClient(**kwargs)
    response = yield http_client.fetch(request)
    body = json.loads(response.body)
    return body


#!/usr/bin/env python
# -*-coding:utf-8-*-
'''
author : shenshuo
date   : 2018年2月6日11:18:40
role   : 解析配置文件
'''

import json
import os
import configparser

def json_to_dict(conf_path):
    if not os.path.isfile(conf_path):
        raise FileNotFoundError('{0} file does not exist'.format(conf_path))

    with open(conf_path, encoding='utf-8') as f:
        try:
            conf_data = json.load(f)
        except ValueError as e:
            raise ValueError('仅解析json格式')
        return conf_data



class IniToDict:
    def __init__(self, conf_path, section):
        self.path = conf_path
        self.sect = section

    def get_option(self, *keys):
        ###判断配置文件path是否存在
        if not os.path.isfile(self.path):
            raise FileNotFoundError('{0} file does not exist'.format(self.path))

        ###解析文件
        fh_conf = configparser.ConfigParser()
        fh_conf.read(self.path)

        ###确认配置文件有无配置层
        list_section = fh_conf.sections()

        res_back = {}
        if self.sect in list_section:
            ###取出section下的所有配置项
            res_list = fh_conf.items(self.sect)

            ###转换成字典
            res_dict = dict(res_list)

            if keys:
                ###取出指定的键
                for k, v in res_dict.items():
                    if k in keys:
                        res_back[k] = v
            else:
                res_back = res_dict
        else:
            raise ValueError('{0} not have {1}'.format(self.path, self.sect))

        ###如果只取一个键，则返回值，其他返回字典
        if keys and len(keys) == 1:
            return res_back.get(keys[0], "")

        return res_back


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Contact : 191715030@qq.com
Author  : shenshuo
Date    : 2018年2月5日13:37:54
Desc    ：记录API
"""


class KerriganAPIS:
    get_publish_config = dict(method='GET',
                         url='/kerrigan/v1/conf/publish/config/',
                         params={'project_code': '',
                                 'environment': '',
                                 'service': 'service',
                                 'filename': 'filename'},
                         field_help={},
                         description='获取配置'
                         )



#!/usr/bin/env python
# -*-coding:utf-8-*-
""""
Contact : 191715030@qq.com
Author : shenshuo
Date   : 2018年2月5日13:37:54
Desc   : 处理API请求
"""

# import jwt
# import shortuuid
from shortuuid import uuid
import traceback
# from .cache import get_cache
from .cache_context import cache_conn
from tornado.web import RequestHandler, HTTPError
from .jwt_token import AuthToken, jwt


class BaseHandler(RequestHandler):
    def __init__(self, *args, **kwargs):
        self.new_csrf_key = str(uuid())
        self.business_id, self.resource_group = None, None
        self.user_id, self.username, self.nickname, self.email, self.is_super = None, None, None, None, False
        self.is_superuser = self.is_super
        self.token_verify = False
        self.params = {}
        super(BaseHandler, self).__init__(*args, **kwargs)

    def get_params_dict(self):
        self.params = {k: self.get_argument(k) for k in self.request.arguments}
        if "auth_key" in self.params: self.params.pop('auth_key')

    def codo_csrf(self):
        # 验证客户端CSRF，如请求为GET，则不验证，否则验证。最后将写入新的key
        cache = cache_conn()

        # or self.request.headers.get('X-Gitlab-Token')
        if self.request.method in ("GET", "HEAD", "OPTIONS") or self.request.headers.get('Sdk-Method'):
            pass
        else:
            csrf_key = self.get_cookie('csrf_key')
            if not csrf_key:  raise HTTPError(402, 'csrf error need csrf key')
            result = cache.get(csrf_key)
            cache.delete(csrf_key)
            if isinstance(result, bytes): result = result.decode()
            if result != '1':   raise HTTPError(402, 'csrf error')
        cache.set(self.new_csrf_key, '1', ex=1800)
        self.set_cookie('csrf_key', self.new_csrf_key)

    def codo_login(self):
        ### 登陆验证
        auth_key = self.get_cookie('auth_key', None)
        if not auth_key:
            url_auth_key = self.get_argument('auth_key', default=None, strip=True)
            if url_auth_key: auth_key = bytes(url_auth_key, encoding='utf-8')

        if not auth_key: raise HTTPError(401, 'auth failed')

        if self.token_verify:
            auth_token = AuthToken()
            user_info = auth_token.decode_auth_token(auth_key)
        else:
            user_info = jwt.decode(auth_key, options={"verify_signature": False}).get('data')

        if not user_info: raise HTTPError(401, 'auth failed')

        self.user_id = user_info.get('user_id', None)
        self.username = user_info.get('username', None)
        self.nickname = user_info.get('nickname', None)
        self.email = user_info.get('email', None)
        self.is_super = user_info.get('is_superuser', False)

        if not self.user_id: raise HTTPError(401, 'auth failed')

        self.user_id = str(self.user_id)
        self.set_secure_cookie("user_id", self.user_id)
        self.set_secure_cookie("nickname", self.nickname)
        self.set_secure_cookie("username", self.username)
        self.set_secure_cookie("email", str(self.email))
        self.is_superuser = self.is_super

    def prepare(self):
        ### 获取url参数为字典
        self.get_params_dict()
        ### 验证客户端CSRF
        self.codo_csrf()

        ### 登陆验证
        self.codo_login()

        # 验证客户端CSRF，如请求为GET，则不验证，否则验证。最后将写入新的key
        # cache = cache_conn()
        # if self.request.method in ("GET", "HEAD", "OPTIONS") or self.request.headers.get('Sdk-Method'):
        #     pass
        # else:
        #     csrf_key = self.get_cookie('csrf_key')
        #     pipeline = cache.get_pipeline()
        #     result = cache.get(csrf_key, private=False, pipeline=pipeline)
        #     cache.delete(csrf_key, private=False, pipeline=pipeline)
        #     if result != '1':
        #         raise HTTPError(402, 'csrf error')
        #
        # cache.set(self.new_csrf_key, 1, expire=1800, private=False)
        # self.set_cookie('csrf_key', self.new_csrf_key)
        #
        # ### 登陆验证
        # auth_key = self.get_cookie('auth_key', None)
        #
        # if not auth_key:
        #     url_auth_key = self.get_argument('auth_key', default=None, strip=True)
        #     if url_auth_key:
        #         auth_key = bytes(url_auth_key, encoding='utf-8')
        #
        # if not auth_key:
        #     # if not auth_key or not self.get_secure_cookie("user_id") or not self.get_secure_cookie("username") :
        #     # 没登录，就让跳到登陆页面
        #     raise HTTPError(401, 'auth failed')
        #
        # else:
        #     if self.token_verify:
        #         auth_token = AuthToken()
        #         user_info = auth_token.decode_auth_token(auth_key)
        #     else:
        #         user_info = jwt.decode(auth_key, verify=False).get('data')
        #     if not user_info:
        #         raise HTTPError(401, 'auth failed')
        #
        #     self.user_id = user_info.get('user_id', None)
        #     self.username = user_info.get('username', None)
        #     self.nickname = user_info.get('nickname', None)
        #     self.email = user_info.get('email', None)
        #     self.is_super = user_info.get('is_superuser', False)
        #
        #     if not self.user_id:
        #         raise HTTPError(401, 'auth failed')
        #     else:
        #         self.user_id = str(self.user_id)
        #         self.set_secure_cookie("user_id", self.user_id)
        #         self.set_secure_cookie("nickname", self.nickname)
        #         self.set_secure_cookie("username", self.username)
        #         self.set_secure_cookie("email", str(self.email))
        # self.is_superuser = self.is_super

    def get_current_user(self):
        return self.username

    def get_current_id(self):
        return self.user_id

    def get_current_nickname(self):
        return self.nickname

    def get_current_email(self):
        return self.email

    def is_superuser(self):
        return self.is_superuser

    @property
    def request_resource_group(self):
        if not self.resource_group:
            resource_group = self.get_secure_cookie("resource_group")
            if resource_group and isinstance(resource_group, bytes):  return bytes.decode(resource_group)
            return None
        return self.resource_group

    @property
    def request_resource_map(self):
        if self.request_resource_group in [None, 'all', '所有项目']:
            return dict()
        else:
            return dict(resource_group=self.request_resource_group)

    @property
    def request_username(self):
        return self.username

    @property
    def request_user_id(self):
        return self.user_id

    @property
    def request_nickname(self):
        return self.nickname

    @property
    def request_email(self):
        return self.email

    @property
    def request_is_superuser(self):
        return self.is_superuser

    def write_error(self, status_code, **kwargs):
        error_trace_list = traceback.format_exception(*kwargs.get("exc_info"))
        if status_code == 404:
            self.set_status(status_code)
            return self.finish('找不到相关路径-404')

        elif status_code == 400:
            self.set_status(status_code)
            return self.finish('bad request...')

        elif status_code == 402:
            self.set_status(status_code)
            return self.finish('csrf error...')

        elif status_code == 403:
            self.set_status(status_code)
            return self.finish('Sorry, you have no permission. Please contact the administrator')

        if status_code == 500:
            self.set_status(status_code)
            for line in error_trace_list:
                self.write(str(line))
            self.finish()

        elif status_code == 401:
            self.set_status(status_code)
            return self.finish('你没有登录')

        else:
            self.set_status(status_code)


class LivenessProbe(RequestHandler):
    def head(self, *args, **kwargs):
        self.write(dict(code=0, msg="I'm OK"))

    def get(self, *args, **kwargs):
        self.write(dict(code=0, msg="I'm OK"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Version : 0.0.1
Contact : 191715030@qq.com
Author  : shenshuo
Date    : 2021/3/2 18:23 
Desc    : 分页
"""

import math
from .model_utils import queryset_to_list


class Page(object):

    def __init__(self, items, page, page_size, total):
        self.items = items
        self.previous_page = None
        self.next_page = None
        self.has_previous = page > 1
        if self.has_previous: self.previous_page = page - 1
        previous_items = (page - 1) * page_size
        self.has_next = previous_items + len(items) < total
        if self.has_next: self.next_page = page + 1
        self.total = total
        self.pages = int(math.ceil(total / float(page_size)))


def paginate(query, order_by: str = None, **query_params):
    page = int(query_params.get('page', 1))
    page_size = int(query_params.get('limit')) if 'limit' in query_params else int(query_params.get('page_size', 10))

    if 'order_by' in query_params: order_by = query_params.get('order_by')

    if page <= 0: raise AttributeError('page needs to be >= 1')
    if page_size <= 0: raise AttributeError('page_size needs to be >= 1')
    if order_by:
        items = query.order_by(order_by).all() if page_size >= 300 else query.limit(page_size).offset(
            (page - 1) * page_size).order_by(order_by).all()
    else:
        items = query.all() if page_size >= 300 else query.limit(page_size).offset((page - 1) * page_size).all()

    total = query.order_by(order_by).count()
    items = queryset_to_list(items)
    return Page(items, page, page_size, total)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""""
Contact : 191715030@qq.com
Author : shenshuo
Date   : 2019年2月5日13:37:54
Desc   : 云厂商的一些方法
"""
from .cloud.qcloud_api import QCloudApiOpt
from .cloud.ucloud_api import UCloudApi


def cloud_factory(cloud):
    if cloud == 'aliyun':
        return None

    elif cloud == 'qcloud':
        return QCloudApiOpt()

    elif cloud == 'ucloud':
        return UCloudApi()

    elif cloud == 'aws':
        return None
    else:
        return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""""
Contact : 191715030@qq.com
Author : shenshuo
Date   : 2019年2月5日13:37:54
Desc   : 云厂商的一些方法
"""

import hmac
import hashlib
import base64
from urllib import parse


class QCloudApiOpt:

    @staticmethod
    def sort_dic(keydict):
        return sorted(zip(keydict.keys(), keydict.values()))

    @staticmethod
    def get_str_sign(sortlist, api_url):
        sign_str_init = ''
        for value in sortlist:
            sign_str_init += value[0] + '=' + value[1] + '&'
        sign_str = 'GET' + api_url + sign_str_init[:-1]
        return sign_str, sign_str_init

    @staticmethod
    def get_signature(sign_str, secret_key):
        secretkey = secret_key
        signature = bytes(sign_str, encoding='utf-8')
        secretkey = bytes(secretkey, encoding='utf-8')
        my_sign = hmac.new(secretkey, signature, hashlib.sha1).digest()
        return base64.b64encode(my_sign)

    @staticmethod
    def encode_signature(my_sign):
        return parse.quote(my_sign)

    @staticmethod
    def get_result_url(sign_str, result_sign, api_url):
        return 'https://' + api_url + sign_str + 'Signature=' + result_sign

    @staticmethod
    def run(keydict, api_url, secret_key):
        sortlist = QCloudApiOpt.sort_dic(keydict)
        # 获取拼接后的sign字符串
        sign_str, sign_str_int = QCloudApiOpt.get_str_sign(sortlist, api_url)
        # 获取签名
        my_sign = QCloudApiOpt.get_signature(sign_str, secret_key)
        # 对签名串进行编码
        result_sign = QCloudApiOpt.encode_signature(my_sign)
        # 获取最终请求url
        result_url = QCloudApiOpt.get_result_url(sign_str_int, result_sign, api_url)
        return result_url



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Contact : 191715030@qq.com
Author  : shenshuo
Date    : 2018年2月5日13:37:54
Desc    ：记录API
"""


class AdminAPIS:
    get_users = dict(method='GET',
                     url='/mg/v2/accounts/user/',
                     params={
                         'page': 1,
                         'limit': 201
                     },
                     field_help={},
                     description='获取用户信息')

    opt_users = dict(method='POST',
                     url='/mg/v2/accounts/user/',
                     body={
                         'username': None,
                         'nickname': None,
                         'password': None,
                         'department': None,
                         'tel': None,
                         'wechat': None,
                         'no': None,
                         'email': None,
                         'user_state': '20',
                     },
                     field_help={
                         'user_state': '20',
                     },
                     description='操作用户数据，支持增删改，请修改method和body数据')

    get_resource_info_by_user = dict(method='GET',
                                     url='/mg/v2/overall/resource/user/',
                                     params={
                                         'nickname': None,  ## 默认为当前用户有权限的
                                         'expand': None  ## yes 是扩展为目录的 ， no 是可以当全局标签的
                                     },
                                     field_help={
                                         'nickname': None,  ## 默认为当前用户有权限的
                                         'expand': None  ## yes 是扩展为目录的 ， no 是可以当全局标签的，没有则是全部
                                     },
                                     description='获取用户有权限的资源组/目录')
    get_resource_info = dict(method='GET',
                             url='/mg/v2/overall/resource/',
                             params={
                                 'key': None,  ## 筛选字段
                                 'value': None  ## 筛选字段值
                             },
                             field_help={
                                 'key': None,  ## 筛选字段
                                 'value': None  ## 筛选字段值
                             },
                             description='获取资源组/目录')



#!/usr/bin/env python
# -*- coding:utf-8 -*-
import jwt, datetime, hashlib
from .configs import configs as my_configs


class AuthToken:
    def __init__(self):
        self.token_secret = my_configs.get('token_secret', '3AIiOq18i~H=WWTIGq4ODQyMzcsIdfghs')

    def encode_auth_token(self, **kwargs):
        """
        生成认证Token
        :param user_id: string
        :param username: string
        :param nickname: string
        :return: string
        """
        try:
            exp_time = kwargs.get('exp_time', 1)
            payload = {
                'exp': datetime.datetime.utcnow() + datetime.timedelta(days=int(exp_time), seconds=10),
                'nbf': datetime.datetime.utcnow() - datetime.timedelta(seconds=10),
                'iat': datetime.datetime.utcnow(),
                'iss': 'auth: ss',
                'sub': 'my token',
                'id': '15618718060',
                'data': {
                    'user_id': kwargs.get('user_id', ''),
                    'username': kwargs.get('username', ''),
                    'nickname': kwargs.get('nickname', ''),
                    'email': kwargs.get('email', ''),
                    'is_superuser': kwargs.get('is_superuser', False)
                }
            }
            return jwt.encode(
                payload,
                self.token_secret,
                algorithm='HS256'
            )

        except Exception as e:
            return e

    def encode_auth_token_v2(self, **kwargs):
        ### 支持到自定义小时
        """
        生成认证Token
        :param user_id: string
        :param username: string
        :param nickname: string
        :param exp_time: int
        :param exp_hour: int
        :return: string
        """
        try:
            exp_days = kwargs.get('exp_days', 1)
            exp_hours = kwargs.get('exp_hours')
            if exp_hours and isinstance(exp_hours, int) and exp_days == 1:
                exp_time =  datetime.datetime.utcnow() + datetime.timedelta(hours=int(exp_hours),  seconds=30)
            else:
                exp_time = datetime.datetime.utcnow() + datetime.timedelta(days=int(exp_days), seconds=30)

            payload = {
                'exp': exp_time,
                'nbf': datetime.datetime.utcnow() - datetime.timedelta(seconds=10),
                'iat': datetime.datetime.utcnow(),
                'iss': 'auth: ss',
                'sub': 'my token',
                'id': '15618718060',
                'data': {
                    'user_id': kwargs.get('user_id', ''),
                    'username': kwargs.get('username', ''),
                    'nickname': kwargs.get('nickname', ''),
                    'email': kwargs.get('email', ''),
                    'is_superuser': kwargs.get('is_superuser', False)
                }
            }
            return jwt.encode(
                payload,
                self.token_secret,
                algorithm='HS256'
            )

        except Exception as e:
            return e

    def decode_auth_token(self, auth_token):
        """
         验证Token
        :param auth_token:
        :return: dict
        """
        try:
            payload = jwt.decode(auth_token, self.token_secret, algorithms=['HS256'],
                                 leeway=datetime.timedelta(seconds=10))
            if 'data' in payload and 'user_id' in payload['data']:
                return payload['data']
            else:
                raise jwt.InvalidTokenError
        except jwt.ExpiredSignatureError:
            return dict(status=-1, msg='Token过期')
        except jwt.InvalidTokenError:
            return dict(status=-2, msg='无效Token')


def gen_md5(pd):
    m2 = hashlib.md5()
    m2.update(pd.encode("utf-8"))
    return m2.hexdigest()



#!/usr/bin/env python
# -*-coding:utf-8-*-
""""
author : shenshuo
date   : 2018年2月5日13:37:54
desc   : OPS SDK
"""

import sys
import os
import re
from io import open
from distutils.core import setup

version = '0.0.50'

if sys.version_info < (2, 7) or (3, 0) <= sys.version_info < (3, 6):
    print('Finit requires at least Python 2.7 or 3.6 to run.')
    sys.exit(1)


# with open(os.path.join('.', '__init__.py'), encoding='utf-8') as f:
#     version = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", f.read(), re.M).group(1)

def get_data_files():
    data_files = [
        ('share/doc/ops_sdk', ['README.md'])
    ]
    return data_files


def get_install_requires():
    requires = ['fire==0.3.1', 'shortuuid==1.0.1', 'pymysql==0.9.3', 'sqlalchemy==1.3.0', 'python3-pika==0.9.14', 'PyJWT==1.7.1',
                'Crypto==1.4.1', 'requests==2.24.0', 'redis==2.10.6', 'tornado>=5.0',
                'aliyun-python-sdk-core-v3==2.13.11', 'aliyun-python-sdk-dysmsapi', 'python-dateutil==2.7.5',
                'ldap3==2.6', 'pydantic==1.7', 'pycryptodome==3.9.9', 'rsa==4.0']
    # if sys.platform.startswith('win'):
    #    requires.append('bottle')
    return requires


setup(
    name='opssdk',
    version=version,
    description="运维SDK",
    packages=['opssdk', 'opssdk.logs', 'opssdk.operate', 'opssdk.install', 'opssdk.get_info', 'opssdk.utils', 'websdk'
        , 'websdk.apis','websdk.cloud','websdk.utils'],
    url='https://github.com/ss1917/ops_sdk/',
    license='GPLv3',
    keywords="ops,opencodo,devops",
    install_requires=get_install_requires(),
    author='shenshuo',
    author_email='191715030@qq.com',
    long_description='SDK of the operation and maintenance script logs operate',
    include_package_data=True,
    data_files=get_data_files(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console :: Curses',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],
    platforms='any'
)



#!/usr/bin/env python
# -*-coding:utf-8-*-
"""
author : shenshuo
date   : 2018年5月22日
role   ：发送短信

pip install --upgrade setuptools
pip install aliyun-python-sdk-core
pip install aliyun-python-sdk-dysmsapi
"""
import sys, json
from libs.consts import const
from aliyunsdkdysmsapi.request.v20170525 import SendSmsRequest
from aliyunsdkcore.client import AcsClient
import uuid
from aliyunsdkcore.profile import region_provider

try:
    reload(sys)
    sys.setdefaultencoding('utf8')
except NameError:
    pass
except Exception as err:
    raise err


class SmsApi:
    def __init__(self, sms_access_key_id, sms_access_key_secret, REGION=const.REGION, DOMAIN=const.DOMAIN,
                 PRODUCT_NAME=const.PRODUCT_NAME):
        self.acs_client = AcsClient(sms_access_key_id, sms_access_key_secret, REGION)
        region_provider.add_endpoint(PRODUCT_NAME, REGION, DOMAIN)

    def send_sms(self, phone_numbers, template_param=None, sign_name="自动化", template_code="SMS_136397944"):
        business_id = uuid.uuid1()
        sms_request = SendSmsRequest.SendSmsRequest()
        # 申请的短信模板编码,必填
        sms_request.set_TemplateCode(template_code)

        # 短信模板变量参数
        if template_param is not None:
            sms_request.set_TemplateParam(template_param)

        # 设置业务请求流水号，必填。
        sms_request.set_OutId(business_id)

        # 短信签名
        sms_request.set_SignName(sign_name)

        # 短信发送的号码列表，必填。
        sms_request.set_PhoneNumbers(phone_numbers)

        # 调用短信发送接口，返回json
        sms_response = self.acs_client.do_action_with_exception(sms_request)

        ##业务处理
        return sms_response


if __name__ == '__main__':
    pass
